import videojs from 'video.js';
import { version as VERSION } from '../package.json';

const Component = videojs.getComponent('Component');

import DownloadButtonControl from './components/downloadButtonControl';
import OpenShareButtonControl from './components/openShareButtonControl';
import ShareMenuButtonsControl from './components/shareMenuButtonsControl';

// Default options for the plugin.
const defaults = {};

// Cross-compatibility for Video.js 5 and 6.
const registerPlugin = videojs.registerPlugin || videojs.plugin;
// const dom = videojs.dom || videojs;

/**
 * Function to invoke when the player is ready.
 *
 * This is a great place for your plugin to initialize itself. When this
 * function is called, the player will have its DOM and child components
 * in place.
 *
 * @function onPlayerReady
 * @param    {Player} player
 *           A Video.js player object.
 *
 * @param    {Object} [options={}]
 *           A plain object containing options for the plugin.
 */
const onPlayerReady = (player, options) => {

  if (!options.shareTypes && !options.downloadButton) {
    return;
  }

  if (player.el().classList.contains('vjs-share-menu')) {
    const playerEl = player.el();

    // clear the container
    const parent = playerEl.querySelector('.vjs-share-menu-container');

    parent.remove();

  } else {
    // first load
    player.addClass('vjs-share-menu');
  }

  const containerComponent = new Component(player);

  containerComponent.addClass('vjs-share-menu-container');

  const topMenuComponent = new Component(player);

  topMenuComponent.addClass('vjs-share-menu-top-container');

  if (options.shareTypes) {
    topMenuComponent.addChild(new OpenShareButtonControl(player, options));
  }

  if (options.downloadButton) {
    topMenuComponent.addChild(new DownloadButtonControl(player, options));
  }

  containerComponent.addChild(topMenuComponent);

  if (options.shareTypes) {
    containerComponent.addChild(new ShareMenuButtonsControl(player, options));
  }

  player.addChild(containerComponent);
};

/**
 * A video.js plugin.
 *
 * In the plugin function, the value of `this` is a video.js `Player`
 * instance. You cannot rely on the player being in a "ready" state here,
 * depending on how the plugin is invoked. This may or may not be important
 * to you; if not, remove the wait for "ready"!
 *
 * @function shareMenu
 * @param    {Object} [options={}]
 *           An object of options left to the plugin author to define.
 */
const shareMenu = function(options) {
  this.ready(() => {
    onPlayerReady(this, videojs.mergeOptions(defaults, options));
  });
};

// Register the plugin with video.js.
registerPlugin('shareMenu', shareMenu);

// Include the version number.
shareMenu.VERSION = VERSION;

export default shareMenu;
